import logging
from unittest import TestCase

from parameterized import parameterized, param

from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestPki(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "pki-integration-test"
    TEST_ROLE = "role-test"

    def setUp(self):
        super().setUp()
        self.client.sys.enable_secrets_engine(
            backend_type="pki",
            path=self.TEST_MOUNT_POINT,
        )
        common_name = "Vault integration tests"
        generate_type = "exported"
        self.client.secrets.pki.generate_root(
            type=generate_type,
            common_name=common_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        name = self.TEST_ROLE
        self.client.secrets.pki.create_or_update_role(
            name=name,
            extra_params={
                "allow_any_name": True,
                "ttl": "6h",
                "max_ttl": "12h",
            },
            mount_point=self.TEST_MOUNT_POINT,
        )

    def tearDown(self):
        self.client.sys.disable_secrets_engine(path=self.TEST_MOUNT_POINT)
        super().tearDown()

    # Read CA Certificate
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_ca_certificate(self, label, raises=False, exception_message=""):
        read_ca_certificate_response = self.client.secrets.pki.read_ca_certificate(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("read_ca_certificate_response: %s" % read_ca_certificate_response)
        self.assertIsInstance(
            obj=read_ca_certificate_response,
            cls=str,
        )

    # Read CA Certificate Chain
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_ca_certificate_chain(self, label, raises=False, exception_message=""):
        read_ca_certificate_chain_response = (
            self.client.secrets.pki.read_ca_certificate_chain(
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(
            "read_ca_certificate_chain_response: %s"
            % read_ca_certificate_chain_response
        )
        self.assertIsInstance(
            obj=read_ca_certificate_chain_response,
            cls=str,
        )

    # Read Certificate
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_certificate(self, label, raises=False, exception_message=""):
        list_certificates_response = self.client.secrets.pki.list_certificates(
            mount_point=self.TEST_MOUNT_POINT,
        )

        serial = list_certificates_response["data"]["keys"][0]
        read_certificate_response = self.client.secrets.pki.read_certificate(
            serial=serial,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("read_certificate_response: %s" % read_certificate_response)
        self.assertIsInstance(
            obj=read_certificate_response,
            cls=dict,
        )

    # List Certificates
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_list_certificates(self, label, raises=False, exception_message=""):
        list_certificates_response = self.client.secrets.pki.list_certificates(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("list_certificates_response: %s" % list_certificates_response)
        self.assertIsInstance(
            obj=list_certificates_response,
            cls=dict,
        )

    # Submit CA Information
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_submit_ca_information(self, label, raises=False, exception_message=""):
        pem_bundle = "{ca_key}{ca_crt}".format(
            ca_key="".join(open(utils.get_config_file_path("ca-key.pem")).readlines()),
            ca_crt="".join(open(utils.get_config_file_path("ca-cert.pem")).readlines()),
        )
        submit_ca_information_response = self.client.secrets.pki.submit_ca_information(
            pem_bundle=pem_bundle,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(
            "submit_ca_information_response: %s" % submit_ca_information_response
        )
        self.assertEqual(
            first=bool(submit_ca_information_response),
            second=True,
        )

    # Read CRL Configuration
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_crl_configuration(self, label, raises=False, exception_message=""):
        expiry = "72h"
        self.client.secrets.pki.set_crl_configuration(
            expiry=expiry,
            mount_point=self.TEST_MOUNT_POINT,
        )

        read_crl_configuration_response = (
            self.client.secrets.pki.read_crl_configuration(
                mount_point=self.TEST_MOUNT_POINT,
            )
        )
        logging.debug(
            "read_crl_configuration_response: %s" % read_crl_configuration_response
        )
        self.assertIsInstance(
            obj=read_crl_configuration_response,
            cls=dict,
        )

    # Set CRL Configuration
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_set_crl_configuration(self, label, raises=False, exception_message=""):
        expiry = "72h"
        disable = False
        set_crl_configuration_response = self.client.secrets.pki.set_crl_configuration(
            expiry=expiry,
            disable=disable,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(
            "set_crl_configuration_response: %s" % set_crl_configuration_response
        )
        self.assertEqual(
            first=bool(set_crl_configuration_response),
            second=True,
        )

    # Read URLs
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_urls(self, label, raises=False, exception_message=""):
        params = {
            "issuing_certificates": ["http://127.0.0.1:8200/v1/pki/ca"],
            "crl_distribution_points": ["http://127.0.0.1:8200/v1/pki/crl"],
        }
        self.client.secrets.pki.set_urls(
            params=params,
            mount_point=self.TEST_MOUNT_POINT,
        )
        read_urls_response = self.client.secrets.pki.read_urls(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("read_urls_response: %s" % read_urls_response)
        self.assertIsInstance(
            obj=read_urls_response,
            cls=dict,
        )

    # Set URLs
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_set_urls(self, label, raises=False, exception_message=""):
        params = {
            "issuing_certificates": ["http://127.0.0.1:8200/v1/pki/ca"],
            "crl_distribution_points": ["http://127.0.0.1:8200/v1/pki/crl"],
        }
        set_urls_response = self.client.secrets.pki.set_urls(
            params=params,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("set_urls_response: %s" % set_urls_response)
        self.assertEqual(
            first=bool(set_urls_response),
            second=True,
        )

    # Read CRL
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_crl(self, label, raises=False, exception_message=""):
        expiry = "72h"
        self.client.secrets.pki.set_crl_configuration(
            expiry=expiry,
            mount_point=self.TEST_MOUNT_POINT,
        )

        read_crl_response = self.client.secrets.pki.read_crl(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("read_crl_response: %s" % read_crl_response)
        self.assertIsInstance(
            obj=read_crl_response,
            cls=str,
        )

    # Rotate CRLs
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_rotate_crl(self, label, raises=False, exception_message=""):
        rotate_crl_response = self.client.secrets.pki.rotate_crl(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("rotate_crl_response: %s" % rotate_crl_response)
        self.assertIsInstance(
            obj=rotate_crl_response,
            cls=dict,
        )

    # Generate Intermediate
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_generate_intermediate(self, label, raises=False, exception_message=""):
        common_name = "Vault integration tests"
        generate_type = "exported"
        generate_intermediate_response = self.client.secrets.pki.generate_intermediate(
            type=generate_type,
            common_name=common_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(
            "generate_intermediate_response: %s" % generate_intermediate_response
        )
        self.assertEqual(
            first=bool(generate_intermediate_response),
            second=True,
        )

    # Set Signed Intermediate
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_set_signed_intermediate(self, label, raises=False, exception_message=""):
        # Generate intermediate CA mount point
        ca_intermediate_pki_mount_point = "{}-signed-intermediate-ca".format(
            self.TEST_MOUNT_POINT
        )
        self.client.sys.enable_secrets_engine(
            backend_type="pki",
            path=ca_intermediate_pki_mount_point,
        )
        # Generate intermediate CA
        common_name = "Vault integration tests intermediate CA"
        generate_type = "exported"
        generate_intermediate_response = self.client.secrets.pki.generate_intermediate(
            type=generate_type,
            common_name=common_name,
            mount_point=ca_intermediate_pki_mount_point,
        )
        logging.debug(
            "generate_intermediate_response: %s" % generate_intermediate_response
        )
        csr = generate_intermediate_response["data"]["csr"]

        # Sign intermediate CA
        sign_intermediate_response = self.client.secrets.pki.sign_intermediate(
            csr=csr,
            common_name=common_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("sign_intermediate_response: %s" % sign_intermediate_response)
        certificate = sign_intermediate_response["data"]["certificate"]

        # Finally test Set Signed Intermediate
        set_signed_intermediate_response = (
            self.client.secrets.pki.set_signed_intermediate(
                certificate=certificate,
                mount_point=ca_intermediate_pki_mount_point,
            )
        )
        logging.debug(
            "set_signed_intermediate_response: %s" % set_signed_intermediate_response
        )
        self.assertEqual(
            first=bool(set_signed_intermediate_response),
            second=True,
        )

        # Now clean intermediate CA
        self.client.sys.disable_secrets_engine(path=ca_intermediate_pki_mount_point)

    # Generate Certificate
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_generate_certificate(self, label, raises=False, exception_message=""):
        name = self.TEST_ROLE
        common_name = "test.example.com"
        generate_certificate_response = self.client.secrets.pki.generate_certificate(
            name=name,
            common_name=common_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(
            "generate_certificate_response: %s" % generate_certificate_response
        )
        self.assertEqual(
            first=bool(generate_certificate_response),
            second=True,
        )

    # Revoke Certificate
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_revoke_certificate(self, label, raises=False, exception_message=""):
        name = self.TEST_ROLE
        common_name = "test.example.com"
        generate_certificate_response = self.client.secrets.pki.generate_certificate(
            name=name,
            common_name=common_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(
            "generate_certificate_response: %s" % generate_certificate_response
        )
        serial_number = generate_certificate_response["data"]["serial_number"]
        logging.debug("serial_number: %s" % serial_number)
        revoke_certificate_response = self.client.secrets.pki.revoke_certificate(
            serial_number=serial_number,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("revoke_certificate_response: %s" % revoke_certificate_response)
        self.assertEqual(
            first=bool(revoke_certificate_response),
            second=True,
        )

    # Create/Update Role
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_create_or_update_role(self, label, raises=False, exception_message=""):
        name = f"{self.TEST_ROLE}-2"
        create_or_update_role_response = self.client.secrets.pki.create_or_update_role(
            name=name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(
            "create_or_update_role_response: %s" % create_or_update_role_response
        )
        self.assertEqual(
            first=bool(create_or_update_role_response),
            second=True,
        )

    # Read Role
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_role(self, label, raises=False, exception_message=""):
        read_role_response = self.client.secrets.pki.read_role(
            name=self.TEST_ROLE,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("read_role_response: %s" % read_role_response)
        self.assertIsInstance(
            obj=read_role_response,
            cls=dict,
        )

    # List Roles
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_list_roles(self, label, raises=False, exception_message=""):
        list_roles_response = self.client.secrets.pki.list_roles(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("list_roles_response: %s" % list_roles_response)
        self.assertIsInstance(
            obj=list_roles_response,
            cls=dict,
        )

    # Delete Role
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_delete_role(self, label, raises=False, exception_message=""):
        name = f"{self.TEST_ROLE}-2"
        self.client.secrets.pki.create_or_update_role(
            name=name,
            mount_point=self.TEST_MOUNT_POINT,
        )

        delete_role_response = self.client.secrets.pki.delete_role(
            name=name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("delete_role_response: %s" % delete_role_response)
        self.assertEqual(
            first=bool(delete_role_response),
            second=True,
        )

    # Generate Root
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_generate_root(self, label, raises=False, exception_message=""):
        ca_pki_mount_point = f"{self.TEST_MOUNT_POINT}-test-ca"
        self.client.sys.enable_secrets_engine(
            backend_type="pki",
            path=ca_pki_mount_point,
        )
        common_name = "Vault integration tests"
        generate_type = "exported"
        generate_root_response = self.client.secrets.pki.generate_root(
            type=generate_type,
            common_name=common_name,
            mount_point=ca_pki_mount_point,
        )
        logging.debug("generate_root_response: %s" % generate_root_response)
        self.assertEqual(
            first=bool(generate_root_response),
            second=True,
        )
        self.client.sys.disable_secrets_engine(path=ca_pki_mount_point)

    # Delete Root
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_delete_root(self, label, raises=False, exception_message=""):
        ca_pki_mount_point = f"{self.TEST_MOUNT_POINT}-test-ca"
        self.client.sys.enable_secrets_engine(
            backend_type="pki",
            path=ca_pki_mount_point,
        )
        common_name = "Vault integration tests"
        generate_type = "exported"
        self.client.secrets.pki.generate_root(
            type=generate_type,
            common_name=common_name,
            mount_point=ca_pki_mount_point,
        )

        delete_root_response = self.client.secrets.pki.delete_root(
            mount_point=ca_pki_mount_point,
        )
        logging.debug("delete_root_response: %s" % delete_root_response)
        self.assertEqual(
            first=bool(delete_root_response),
            second=True,
        )
        self.client.sys.disable_secrets_engine(path=ca_pki_mount_point)

    # Sign Intermediate
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_sign_intermediate(self, label, raises=False, exception_message=""):
        # Generate intermediate CA
        ca_intermediate_pki_mount_point = "{}-intermediate-ca".format(
            self.TEST_MOUNT_POINT
        )
        self.client.sys.enable_secrets_engine(
            backend_type="pki",
            path=ca_intermediate_pki_mount_point,
        )
        common_name = "Vault integration tests intermediate CA"
        generate_type = "exported"
        generate_intermediate_response = self.client.secrets.pki.generate_intermediate(
            type=generate_type,
            common_name=common_name,
            mount_point=ca_intermediate_pki_mount_point,
        )
        logging.debug(
            "generate_intermediate_response: %s" % generate_intermediate_response
        )
        csr = generate_intermediate_response["data"]["csr"]

        sign_intermediate_response = self.client.secrets.pki.sign_intermediate(
            csr=csr,
            common_name=common_name,
            extra_params={"use_csr_values": True},
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("sign_intermediate_response: %s" % sign_intermediate_response)
        self.assertEqual(
            first=bool(sign_intermediate_response),
            second=True,
        )
        # Now clean intermediate CA
        self.client.sys.disable_secrets_engine(path=ca_intermediate_pki_mount_point)

    # Sign Self-Issued
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_sign_self_issued(self, label, raises=False, exception_message=""):
        sign_self_issued_response = self.client.secrets.pki.sign_self_issued(
            certificate="".join(
                open(utils.get_config_file_path("ca-cert.pem")).readlines()
            ),
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("sign_self_issued_response: %s" % sign_self_issued_response)
        self.assertEqual(
            first=bool(sign_self_issued_response),
            second=True,
        )

    # Sign Certificate
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_sign_certificate(self, label, raises=False, exception_message=""):
        name = self.TEST_ROLE
        common_name = "another.example.com"
        sign_certificate_response = self.client.secrets.pki.sign_certificate(
            name=name,
            csr="".join(
                open(utils.get_config_file_path("server-cert.csr")).readlines()
            ),
            common_name=common_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("sign_certificate_response: %s" % sign_certificate_response)
        self.assertEqual(
            first=bool(sign_certificate_response),
            second=True,
        )

    # Sign Verbatim
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_sign_verbatim(self, label, raises=False, exception_message=""):
        name = self.TEST_ROLE
        sign_verbatim_response = self.client.secrets.pki.sign_verbatim(
            csr="".join(
                open(utils.get_config_file_path("server-cert.csr")).readlines()
            ),
            name=name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("sign_verbatim_response: %s" % sign_verbatim_response)
        self.assertEqual(
            first=bool(sign_verbatim_response),
            second=True,
        )

    # Tidy
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_tidy(self, label, raises=False, exception_message=""):
        tidy_response = self.client.secrets.pki.tidy(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("tidy_response: %s" % tidy_response)
        self.assertEqual(
            first=bool(tidy_response),
            second=True,
        )



#!/usr/bin/env python
import logging
import os
import subprocess
import time
import requests
import hcl

import distutils.spawn
from unittest import SkipTest
from tests.utils import get_config_file_path, load_config_file, create_client

logger = logging.getLogger(__name__)


class ServerManager:
    """Runs vault process running with test configuration and associates a hvac Client instance with this process."""

    def __init__(self, config_paths, client, use_consul=False):
        """Set up class attributes for managing a vault server process.

        :param config_paths: Full path to the Vault config to use when launching `vault server`.
        :type config_paths: list[str]
        :param client: Hvac Client that is used to initialize the vault server process.
        :type client: hvac.v1.Client
        """
        self.config_paths = config_paths
        self.client = client
        self.use_consul = use_consul

        self.keys = None
        self.root_token = None

        self._processes = []

    def start(self):
        """Launch the vault server process and wait until its online and ready."""
        if self.use_consul:
            self.start_consul()

        if distutils.spawn.find_executable("vault") is None:
            raise SkipTest("Vault executable not found")

        # If a vault server is already running then we won't be able to start another one.
        # If we can't start our vault server then we don't know what we're testing against.
        try:
            self.client.sys.is_initialized()
        except Exception:
            pass
        else:
            raise Exception("Vault server already running")

        cluster_ready = False
        for config_path in self.config_paths:
            command = ["vault", "server", "-config=" + config_path]
            logger.debug(f"Starting vault server with command: {command}")
            process = subprocess.Popen(
                command, stdout=subprocess.PIPE, stderr=subprocess.PIPE
            )
            self._processes.append(process)
            logger.debug(f"Spawned vault server with PID {process.pid}")

            attempts_left = 20
            last_exception = None
            while attempts_left > 0 and not cluster_ready:
                try:
                    logger.debug("Checking if vault is ready...")
                    self.client.sys.is_initialized()
                    cluster_ready = True
                    break
                except Exception as ex:
                    if process.poll() is not None:
                        raise Exception("Vault server terminated before becoming ready")
                    logger.debug("Waiting for Vault to start")
                    time.sleep(0.5)
                    attempts_left -= 1
                    last_exception = ex
            if not cluster_ready:
                if process.poll() is None:
                    process.kill()
                stdout, stderr = process.communicate()
                raise Exception(
                    "Unable to start Vault in background:\n{err}\n{stdout}\n{stderr}".format(
                        err=last_exception,
                        stdout=stdout,
                        stderr=stderr,
                    )
                )

    def start_consul(self):
        if distutils.spawn.find_executable("consul") is None:
            raise SkipTest("Consul executable not found")

        try:
            requests.get("http://127.0.0.1:8500/v1/catalog/nodes")
        except Exception:
            pass
        else:
            raise Exception("Consul service already running")

        command = ["consul", "agent", "-dev"]
        logger.debug(f"Starting consul service with command: {command}")
        process = subprocess.Popen(
            command, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        self._processes.append(process)
        attempts_left = 20
        last_exception = None
        while attempts_left > 0:
            try:
                catalog_nodes_response = requests.get(
                    "http://127.0.0.1:8500/v1/catalog/nodes"
                )
                nodes_list = catalog_nodes_response.json()
                logger.debug(
                    "JSON response from request to consul/v1/catalog/noses: {resp}".format(
                        resp=nodes_list
                    )
                )
                node_name = nodes_list[0]["Node"]
                logger.debug(f"Current consul node name: {node_name}")
                node_health_response = requests.get(
                    f"http://127.0.0.1:8500/v1/health/node/{node_name}"
                )
                node_health = node_health_response.json()
                logger.debug(f"Node health response: {node_health}")
                assert (
                    node_health[0]["Status"] == "passing"
                ), f'Node {node_name} status != "passing"'
                return True
            except Exception as error:
                if process.poll() is not None:
                    raise Exception("Consul service terminated before becoming ready")
                logger.debug(
                    "Unable to connect to consul while waiting for process to start: {err}".format(
                        err=error
                    )
                )
                time.sleep(0.5)
                attempts_left -= 1
                last_exception = error

        raise Exception(f"Unable to start consul in background: {last_exception}")

    def stop(self):
        """Stop the vault server process being managed by this class."""
        for process_num, process in enumerate(self._processes):
            logger.debug(f"Terminating vault server with PID {process.pid}")
            if process.poll() is None:
                process.kill()
            if os.getenv("HVAC_OUTPUT_VAULT_STDERR", False):
                stdout_lines, stderr_lines = process.communicate()
                stderr_filename = f"vault{process_num}_stderr.log"
                with open(get_config_file_path(stderr_filename), "w") as f:
                    logger.debug(stderr_lines.decode())
                    f.writelines(stderr_lines.decode())
                stdout_filename = f"vault{process_num}_stdout.log"
                with open(get_config_file_path(stdout_filename), "w") as f:
                    logger.debug(stdout_lines.decode())
                    f.writelines(stdout_lines.decode())

    def initialize(self):
        """Perform initialization of the vault server process and record the provided unseal keys and root token."""
        assert not self.client.sys.is_initialized()

        result = self.client.sys.initialize()

        self.root_token = result["root_token"]
        self.keys = result["keys"]

    def restart_vault_cluster(self, perform_init=True):
        self.stop()
        self.start()
        if perform_init:
            self.initialize()

    def get_active_vault_addresses(self):
        vault_addresses = []
        for config_path in self.config_paths:
            config_hcl = load_config_file(config_path)
            config = hcl.loads(config_hcl)
            try:
                vault_address = "https://{addr}".format(
                    addr=config["listener"]["tcp"]["address"]
                )
            except KeyError as error:
                logger.debug(
                    "Unable to find explict Vault address in config file {path}: {err}".format(
                        path=config_path,
                        err=error,
                    )
                )
                vault_address = "https://127.0.0.1:8200"
                logger.debug(f"Using default address: {vault_address}")
            vault_addresses.append(vault_address)
        return vault_addresses

    def unseal(self):
        """Unseal the vault server process."""
        vault_addresses = self.get_active_vault_addresses()
        for vault_address in vault_addresses:
            create_client(url=vault_address).sys.submit_unseal_keys(self.keys)



from unittest import TestCase

from unittest.mock import create_autospec

from hvac.adapters import Adapter
from hvac.api.vault_api_base import VaultApiBase
from hvac.api.vault_api_category import VaultApiCategory

UNIMPLEMENTED_CLASSES = ["not_implemented"]


class ImplementedVaultApiBase(VaultApiBase):
    pass


class ImplementedVaultApiCategory(VaultApiCategory):
    @property
    def implemented_classes(self):
        return [ImplementedVaultApiBase]

    @property
    def unimplemented_classes(self):
        return UNIMPLEMENTED_CLASSES


class TestVaultApiCategory(TestCase):
    mock_vault_api_category = None
    mock_adapter = None

    def setUp(self):
        self.mock_adapter = create_autospec(Adapter)
        self.mock_vault_api_category = ImplementedVaultApiCategory(
            adapter=self.mock_adapter,
        )

    def tearDown(self):
        self.mock_vault_api_category = None
        self.mock_adapter = None

    def test_getattr(self):
        assert self.mock_vault_api_category.implementedvaultapibase

        with self.assertRaises(AttributeError):
            assert self.mock_vault_api_category.not_an_attribute

        with self.assertRaises(AttributeError):
            assert self.mock_vault_api_category.not_an_attribute.on_not_an_attribute

        with self.assertRaises(NotImplementedError):
            assert self.mock_vault_api_category.not_implemented

    def test_unimplemented_classes(self):
        self.assertEqual(
            first=self.mock_vault_api_category.unimplemented_classes,
            second=UNIMPLEMENTED_CLASSES,
        )

    def test_adapter_property(self):
        self.assertIsInstance(
            obj=self.mock_vault_api_category.adapter,
            cls=Adapter,
        )

        new_adapter = "this is my new adapter"
        self.mock_vault_api_category.adapter = new_adapter

        self.assertEqual(
            first=self.mock_vault_api_category._implementedvaultapibase.adapter,
            second=new_adapter,
        )



#!/usr/bin/env python
import logging
from unittest import TestCase

import requests_mock
from parameterized import parameterized, param

from hvac.adapters import JSONAdapter
from hvac.api.secrets_engines import Aws
from hvac.api.secrets_engines.aws import DEFAULT_MOUNT_POINT
from hvac.exceptions import ParamValidationError


class TestAws(TestCase):
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_rotate_root_iam_credentials(
        self, test_label, mount_point=DEFAULT_MOUNT_POINT
    ):
        expected_status_code = 200
        mock_response = {"data": {"access_key": "AKIA..."}}
        aws = Aws(adapter=JSONAdapter())
        mock_url = "http://localhost:8200/v1/{mount_point}/config/rotate-root".format(
            mount_point=mount_point,
        )
        logging.debug("Mocking URL: %s" % mock_url)
        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method="POST",
                url=mock_url,
                status_code=expected_status_code,
                json=mock_response,
            )
            rotate_root_response = aws.rotate_root_iam_credentials(
                mount_point=mount_point,
            )
        logging.debug("rotate_root_response: %s" % rotate_root_response)
        self.assertEqual(
            first=mock_response,
            second=rotate_root_response,
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "invalid endpoint",
                endpoint="cats",
                raises=ParamValidationError,
                exception_msg="cats",
            ),
        ]
    )
    def test_generate_credentials(
        self,
        test_label,
        role_name="hvac-test-role",
        mount_point=DEFAULT_MOUNT_POINT,
        endpoint="creds",
        raises=None,
        exception_msg="",
    ):
        expected_status_code = 200
        mock_response = {
            "data": {
                "access_key": "AKIA...",
                "secret_key": "xlCs...",
                "security_token": None,
            }
        }
        mock_url = "http://localhost:8200/v1/{mount_point}/creds/{role_name}".format(
            mount_point=mount_point,
            role_name=role_name,
        )
        logging.debug("Mocking URL: %s" % mock_url)
        aws = Aws(adapter=JSONAdapter())
        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method="GET",
                url=mock_url,
                status_code=expected_status_code,
                json=mock_response,
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    aws.generate_credentials(
                        name=role_name,
                        endpoint=endpoint,
                        mount_point=mount_point,
                    )
                self.assertIn(
                    member=exception_msg,
                    container=str(cm.exception),
                )
            else:
                gen_creds_response = aws.generate_credentials(
                    name=role_name,
                    endpoint=endpoint,
                    mount_point=mount_point,
                )
                logging.debug("gen_creds_response: %s" % gen_creds_response)
                self.assertEqual(
                    first=mock_response,
                    second=gen_creds_response,
                )



# Configuration file for the Sphinx documentation builder.


# -- Path setup --------------------------------------------------------------
# Set up import path to allow the autodoc extension to find the local module code.
import os
import sys

sys.path.insert(0, os.path.abspath(".."))


# -- Project information -----------------------------------------------------

project = "hvac"
copyright = "2018-2020, Ian Unruh, Jeffrey Hogan"
author = "Ian Unruh, Jeffrey Hogan"

# The short X.Y version
version = "1.2.0"
# The full version, including alpha/beta/rc tags
release = "1.2.0"


# -- General configuration ---------------------------------------------------

extensions = [
    "docs.ext.hvac_doctest",
    "sphinx.ext.autodoc",
    "sphinx.ext.coverage",
    "sphinx.ext.viewcode",
    "sphinx.ext.githubpages",
    "m2r2",
    "autodocsumm",
]

# https://github.com/CrossNox/m2r2/blob/0408d7acea843485d9ff42ee08a105a79f045493/m2r2.py#L675C27-L675C51
# https://github.com/CrossNox/m2r2/issues/30
# We use m2r primarily to convert the markdown changelog to RST, so we don't need named references.
# Since we may have multiple changelog entries refer to the same GitHub issue, and we use the same text
# to anchor it (GH-###), it would result in duplicate explicit target names.
m2r_anonymous_references = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

source_suffix = [".rst", ".md"]

# The master toctree document.
master_doc = "index"

language = None
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]
pygments_style = "sphinx"


# -- Options for HTML output -------------------------------------------------

html_theme = "sphinx_rtd_theme"
html_static_path = ["_static"]
html_context = {"no_skippy": True}
html_theme_options = {
    # Toc options
    "collapse_navigation": False,
}

# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = "hvacdoc"

# -- Options for Epub output -------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project
epub_author = author
epub_publisher = author
epub_copyright = copyright

# A list of files that should not be packed into the epub file.
epub_exclude_files = ["search.html"]

# -- doctest configuration -------------------------------------------------
if os.getenv("READ_THE_DOCS_BUILD") is not None:
    doctest_global_enabled = False

doctest_global_setup = """
import os
from pprint import pprint, pformat

import mock

import hvac
from tests import utils as test_utils
from tests.doctest import doctest_global_setup
from tests.utils.server_manager import ServerManager

client_cert_path = test_utils.get_config_file_path('client-cert.pem')
client_key_path = test_utils.get_config_file_path('client-key.pem')
server_cert_path = test_utils.get_config_file_path('server-cert.pem')

manager, mocker = doctest_global_setup()
client = manager.client
"""

doctest_global_cleanup = """
mocker.stop()
manager.stop()
"""

# -- Autodoc configuration -------------------------------------------------

autodoc_default_options = {
    "autosummary": True,
}


def skip(app, what, name, obj, skip, options):
    """Method to override default autodoc skip call. Ensures class constructor
    (e.g., __init__()) methods are included regardless of if private methods
    are included in the documentation generally.
    """
    if name == "__init__":
        return False
    return skip


def setup(app):
    app.connect("autodoc-skip-member", skip)



from unittest import TestCase

import requests_mock

from hvac import Client


class TestAuthMethods(TestCase):
    """Tests for methods related to Vault sys/auth routes."""

    @requests_mock.Mocker()
    def test_tune_auth_backend(self, requests_mocker):
        expected_status_code = 204
        test_mount_point = "approle-test"
        test_description = "this is a test description"
        requests_mocker.register_uri(
            method="POST",
            url=f"http://localhost:8200/v1/sys/auth/{test_mount_point}/tune",
            status_code=expected_status_code,
        )
        client = Client()
        actual_response = client.sys.tune_auth_method(
            path=test_mount_point,
            description=test_description,
        )

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

        actual_request_params = requests_mocker.request_history[0].json()

        # Ensure we sent through an optional tune parameter as expected
        self.assertEqual(
            first=test_description,
            second=actual_request_params["description"],
        )

    @requests_mock.Mocker()
    def test_get_auth_backend_tuning(self, requests_mocker):
        expected_status_code = 200
        test_mount_point = "approle-test"
        mock_response = {
            "max_lease_ttl": 12345678,
            "lease_id": "",
            "force_no_cache": False,
            "warnings": None,
            "data": {
                "force_no_cache": False,
                "default_lease_ttl": 2764800,
                "max_lease_ttl": 12345678,
            },
            "wrap_info": None,
            "auth": None,
            "lease_duration": 0,
            "request_id": "673f2336-3235-b988-2194-c68261a02bfe",
            "default_lease_ttl": 2764800,
            "renewable": False,
        }
        requests_mocker.register_uri(
            method="GET",
            url=f"http://localhost:8200/v1/sys/auth/{test_mount_point}/tune",
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        actual_response = client.sys.read_auth_method_tuning(
            path=test_mount_point,
        )

        self.assertEqual(
            first=mock_response,
            second=actual_response,
        )



#!/usr/bin/env python
import pytest
import logging
from unittest import TestCase
from unittest import mock

import requests_mock
from requests_mock.response import create_response
from parameterized import parameterized, param
from hvac.constants.client import DEFAULT_URL
from hvac import exceptions
from hvac import adapters


class TestAdapters:
    CONSTRUCTOR_ARGS = (
        "base_uri",
        "token",
        "cert",
        "verify",
        "timeout",
        "proxies",
        "allow_redirects",
        "session",
        "namespace",
        "ignore_exceptions",
        "strict_http",
        "request_header",
    )

    INTERNAL_KWARGS = (
        "cert",
        "verify",
        "timeout",
        "proxies",
    )

    @pytest.mark.parametrize(
        "conargs",
        [
            {arg: arg.capitalize() for arg in CONSTRUCTOR_ARGS},
            {arg: arg.upper() for arg in CONSTRUCTOR_ARGS},
        ],
    )
    def test_from_adapter(self, conargs):
        expected = conargs.copy()
        for internal_kwarg in self.INTERNAL_KWARGS:
            expected.setdefault("_kwargs", {})[internal_kwarg] = expected.pop(
                internal_kwarg
            )

        # let's start with a JSONAdapter, and make a RawAdapter out of it
        json_adapter = adapters.JSONAdapter(**conargs)

        raw_adapter = adapters.RawAdapter.from_adapter(json_adapter)

        for property, value in expected.items():
            assert getattr(raw_adapter, property) == value


class TestRequest(TestCase):
    """Unit tests providing coverage for requests-related methods in the hvac Client class."""

    @parameterized.expand(
        [
            param(
                "standard Vault address",
                url="https://localhost:8200",
            ),
            param(
                "Vault address with route",
                url="https://example.com/vault",
            ),
            param(
                "regression test for hvac issue #51",
                url="https://localhost:8200",
                path="keyring/http://some.url/sub/entry",
            ),
            param(
                "redirect with location header for issue #343",
                url="https://localhost:8200",
                path="secret/some-secret",
                redirect_url="https://some-other-place.com/secret/some-secret",
            ),
        ]
    )
    def test_get(self, label, url, path="v1/sys/health", redirect_url=None):
        path = path.replace("//", "/")
        expected_status_code = 200
        mock_url = f"{url}/{path}"
        expected_request_urls = [mock_url]
        adapter = adapters.RawAdapter(base_uri=url)
        response_headers = {}
        response_status_code = 200
        if redirect_url is not None:
            response_headers["Location"] = redirect_url
            response_status_code = 301
        with requests_mock.mock() as requests_mocker:
            logging.debug('Registering "mock_url": %s' % mock_url)
            requests_mocker.register_uri(
                method="GET",
                url=mock_url,
                headers=response_headers,
                status_code=response_status_code,
            )
            if redirect_url is not None:
                expected_request_urls.append(redirect_url)
                logging.debug('Registering "redirect_url": %s' % redirect_url)
                requests_mocker.register_uri(
                    method="GET",
                    url=redirect_url,
                )

            response = adapter.get(
                url=path,
            )

        # Assert all our expected uri(s) were requested
        for request_num, expected_request_url in enumerate(expected_request_urls):
            self.assertEqual(
                first=expected_request_url,
                second=requests_mocker.request_history[request_num].url,
            )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("kv secret lookup", "v1/secret/some-secret"),
        ]
    )
    @requests_mock.Mocker()
    def test_list(self, test_label, test_path, requests_mocker):
        mock_response = {
            "auth": None,
            "data": {"keys": ["things1", "things2"]},
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "ba933afe-84d4-410f-161b-592a5c016009",
            "warnings": None,
            "wrap_info": None,
        }
        expected_status_code = 200
        mock_url = f"{DEFAULT_URL}/{test_path}"
        requests_mocker.register_uri(method="LIST", url=mock_url, json=mock_response)
        adapter = adapters.RawAdapter()
        response = adapter.list(
            url=test_path,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )
        self.assertEqual(first=mock_response, second=response.json())


@pytest.fixture
def raw_adapter():
    return mock.Mock(wraps=adapters.RawAdapter())


class TestRawAdapter:
    @pytest.mark.parametrize("headers", [{}, {"Content-Type": "application/json"}])
    @pytest.mark.parametrize("json_get", [None, Exception])
    @pytest.mark.parametrize("text_get", [None, Exception])
    @pytest.mark.parametrize(
        "bytes", [None, b"", b"a", b"{}", b'{"errors": ["err"]}', b"\x80\x81", b"\0"]
    )
    @pytest.mark.parametrize("code", [404, 500])
    def test_raise_for_error(
        self, raw_adapter, headers, bytes, code, json_get, text_get
    ):
        url = "throwaway"
        method = "GET"

        resp = create_response(
            mock.Mock(url=url), status_code=code, content=bytes, headers=headers
        )

        resp.json = mock.Mock(wraps=resp.json, side_effect=json_get)

        mock_text = mock.PropertyMock(wraps=resp.text, side_effect=text_get)
        with mock.patch("requests.Response.text", new=mock_text):
            if text_get is not None:
                with pytest.raises(Exception):
                    resp.text

            text = errors = json = None

            if headers:
                try:
                    json = resp.json()
                except Exception:
                    pass
                else:
                    errors = json.get("errors")

            try:
                text = resp.text
            except Exception:
                pass

            from hvac.utils import raise_for_error

            with mock.patch(
                "hvac.utils.raise_for_error", mock.Mock(wraps=raise_for_error)
            ) as r:
                with pytest.raises(exceptions.VaultError) as e:
                    raw_adapter._raise_for_error(method, url, resp)

                e_msg = None if errors else text
                expected = mock.call(
                    method, url, code, e_msg, errors=errors, text=text, json=json
                )

                assert r.call_count == 1
                r.assert_has_calls([expected])
                assert e.value.text == text
                assert e.value.json == json
                assert e.value.errors == errors



from contextlib import contextmanager
from unittest import TestCase
from hvac import exceptions

from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestToken(HvacIntegrationTestCase, TestCase):
    # would rather these be pytest fixtures
    @contextmanager
    def prep_policy(self, name):
        try:
            yield (name, self.prep_policy(name))
        finally:
            self.client.sys.delete_policy(name)

    @contextmanager
    def prep_role(self, name, policies=None):
        role = self.client.auth.token.create_or_update_role(
            name, allowed_policies=policies
        )
        assert role.status_code == 204
        try:
            yield (name, role, policies)
        finally:
            self.client.auth.token.delete_role(name)

    @contextmanager
    def test_policy(self):
        with self.prep_policy(["testpolicy"]) as p:
            yield p

    @contextmanager
    def test_role(self):
        with self.test_policy() as p, self.prep_role(
            name="testrole", policies=p[0]
        ) as r:
            yield r

    def test_auth_token_manipulation(self):
        result = self.client.auth.token.create(ttl="1h", renewable=True)
        assert result["auth"]["client_token"]

        lookup = self.client.auth.token.lookup(result["auth"]["client_token"])
        assert result["auth"]["client_token"] == lookup["data"]["id"]

        renew = self.client.auth.token.renew(lookup["data"]["id"])
        assert result["auth"]["client_token"] == renew["auth"]["client_token"]

        self.client.auth.token.revoke(lookup["data"]["id"])

        try:
            lookup = self.client.auth.token.lookup(result["auth"]["client_token"])
            assert False
        except exceptions.Forbidden:
            assert True
        except exceptions.InvalidPath:
            assert True
        except exceptions.InvalidRequest:
            assert True

    def test_self_auth_token_manipulation(self):
        result = self.client.auth.token.create(ttl="1h", renewable=True)
        assert result["auth"]["client_token"]
        self.client.token = result["auth"]["client_token"]

        lookup = self.client.auth.token.lookup_self()
        assert result["auth"]["client_token"] == lookup["data"]["id"]

        renew = self.client.auth.token.renew_self()
        assert result["auth"]["client_token"] == renew["auth"]["client_token"]

        self.client.auth.token.revoke_self()

        try:
            lookup = self.client.auth.token.lookup(result["auth"]["client_token"])
            assert False
        except exceptions.Forbidden:
            assert True
        except exceptions.InvalidPath:
            assert True
        except exceptions.InvalidRequest:
            assert True

    def test_auth_orphaned_token_manipulation(self):
        result = self.client.auth.token.create_orphan(ttl="1h", renewable=True)
        assert result["auth"]["client_token"]

        lookup = self.client.auth.token.lookup(result["auth"]["client_token"])
        assert result["auth"]["client_token"] == lookup["data"]["id"]

        renew = self.client.auth.token.renew(lookup["data"]["id"])
        assert result["auth"]["client_token"] == renew["auth"]["client_token"]

        self.client.auth.token.revoke(lookup["data"]["id"])

        try:
            lookup = self.client.auth.token.lookup(result["auth"]["client_token"])
            assert False
        except exceptions.Forbidden:
            assert True
        except exceptions.InvalidPath:
            assert True
        except exceptions.InvalidRequest:
            assert True

    def test_token_accessor(self):
        # Create token, check accessor is provided
        result = self.client.auth.token.create(ttl="1h")
        token_accessor = result["auth"].get("accessor", None)
        assert token_accessor

        # Look up token by accessor, make sure token is excluded from results
        lookup = self.client.auth.token.lookup_accessor(token_accessor)
        assert lookup["data"]["accessor"] == token_accessor
        assert not lookup["data"]["id"]

        # Revoke token using the accessor
        self.client.auth.token.revoke_accessor(token_accessor)

        # Look up by accessor should fail
        with self.assertRaises(exceptions.InvalidRequest):
            lookup = self.client.auth.token.lookup_accessor(token_accessor)

        # As should regular lookup
        with self.assertRaises(exceptions.Forbidden):
            lookup = self.client.auth.token.lookup(result["auth"]["client_token"])

    def test_create_token_explicit_max_ttl(self):

        token = self.client.auth.token.create(ttl="30m", explicit_max_ttl="5m")

        assert token["auth"]["client_token"]

        assert token["auth"]["lease_duration"] == 300

        # Validate token
        lookup = self.client.auth.token.lookup(token["auth"]["client_token"])
        assert token["auth"]["client_token"] == lookup["data"]["id"]

    def test_create_token_max_ttl(self):

        token = self.client.auth.token.create(ttl="5m")

        assert token["auth"]["client_token"]

        assert token["auth"]["lease_duration"] == 300

        # Validate token
        lookup = self.client.auth.token.lookup(token["auth"]["client_token"])
        assert token["auth"]["client_token"] == lookup["data"]["id"]

    def test_create_token_periodic(self):

        token = self.client.auth.token.create(period="30m")

        assert token["auth"]["client_token"]

        assert token["auth"]["lease_duration"] == 1800

        # Validate token
        lookup = self.client.auth.token.lookup(token["auth"]["client_token"])
        assert token["auth"]["client_token"] == lookup["data"]["id"]
        assert lookup["data"]["period"] == 1800

    def test_create_wrapped_token_periodic(self):

        response = self.client.auth.token.create(period="30m", wrap_ttl="15m")

        assert "wrap_info" in response, repr(response)
        assert response["wrap_info"] is not None, repr(response)
        assert response["auth"] is None, repr(response)
        assert response["wrap_info"]["ttl"] == 900
        assert "token" in response["wrap_info"]

        # unwrap
        token = self.client.sys.unwrap(token=response["wrap_info"]["token"])

        assert token["auth"]["client_token"]
        assert token["auth"]["lease_duration"] == 1800

        # Validate token
        lookup = self.client.auth.token.lookup(token["auth"]["client_token"])
        assert token["auth"]["client_token"] == lookup["data"]["id"]
        assert lookup["data"]["period"] == 1800

    def test_token_roles(self):
        # No roles, list_token_roles == None
        with self.assertRaises(exceptions.InvalidPath):
            self.client.auth.token.list_roles()

        try:
            # Create token role
            assert (
                self.client.auth.token.create_or_update_role("testrole").status_code
                == 204
            )

            # List token roles
            during = self.client.auth.token.list_roles()["data"]["keys"]
            assert len(during) == 1
            assert during[0] == "testrole"

        finally:
            # Delete token role
            self.client.auth.token.delete_role("testrole")

        # No roles, list_token_roles == None
        with self.assertRaises(exceptions.InvalidPath):
            self.client.auth.token.list_roles()

    def test_create_token_w_role(self):
        with self.test_role() as test_role:
            role_name, _, policies = test_role
            expected_policies = ["default"] + policies

            # Create token against role
            token = self.client.auth.token.create(ttl="1h", role_name=role_name)
            assert token["auth"]["client_token"]
            assert token["auth"]["policies"] == expected_policies

    def test_create_wrapped_token_w_role(self):
        with self.test_role() as test_role:
            role_name, _, policies = test_role
            expected_policies = ["default"] + policies

            # Create token against role
            response = self.client.auth.token.create(
                ttl="1h", role_name=role_name, wrap_ttl="15m"
            )

            assert "wrap_info" in response, repr(response)
            assert response["wrap_info"] is not None, repr(response)
            assert response["auth"] is None, repr(response)
            assert response["wrap_info"]["ttl"] == 900
            assert "token" in response["wrap_info"]

            # unwrap
            token = self.client.sys.unwrap(token=response["wrap_info"]["token"])
            assert token["auth"]["client_token"]
            assert token["auth"]["policies"] == expected_policies



import logging
from unittest import TestCase, skipIf

from hvac import exceptions
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class IntegrationTest(HvacIntegrationTestCase, TestCase):
    def setUp(self):
        super().setUp()
        if "secret/" not in self.client.sys.list_mounted_secrets_engines()["data"]:
            self.client.sys.enable_secrets_engine(
                backend_type="kv",
                path="secret",
                options=dict(version=1),
            )

    def test_generic_secret_backend(self):
        self.client.write("secret/foo", zap="zip")
        result = self.client.read("secret/foo")

        assert result["data"]["zap"] == "zip"

        self.client.delete("secret/foo")

    def test_list_directory(self):
        self.client.write("secret/test-list/bar/foo", value="bar")
        self.client.write("secret/test-list/foo", value="bar")
        result = self.client.list("secret/test-list")

        assert result["data"]["keys"] == ["bar/", "foo"]

        self.client.delete("secret/test-list/bar/foo")
        self.client.delete("secret/test-list/foo")

    def test_write_with_response(self):
        if "transit/" in self.client.sys.list_mounted_secrets_engines()["data"]:
            self.client.sys.disable_secrets_engine("transit")
        self.client.sys.enable_secrets_engine("transit")

        plaintext = "test"

        self.client.write("transit/keys/foo")

        result = self.client.write("transit/encrypt/foo", plaintext=plaintext)
        ciphertext = result["data"]["ciphertext"]

        result = self.client.write("transit/decrypt/foo", ciphertext=ciphertext)
        assert result["data"]["plaintext"] == plaintext

    def test_read_nonexistent_key(self):
        assert not self.client.read("secret/I/dont/exist")

    def test_auth_token_manipulation(self):
        result = self.client.auth.token.create(ttl="1h", renewable=True)
        assert result["auth"]["client_token"]

        lookup = self.client.lookup_token(result["auth"]["client_token"])
        assert result["auth"]["client_token"] == lookup["data"]["id"]

        renew = self.client.renew_token(lookup["data"]["id"])
        assert result["auth"]["client_token"] == renew["auth"]["client_token"]

        self.client.revoke_token(lookup["data"]["id"])

        try:
            lookup = self.client.lookup_token(result["auth"]["client_token"])
            assert False
        except exceptions.Forbidden:
            assert True
        except exceptions.InvalidPath:
            assert True
        except exceptions.InvalidRequest:
            assert True

    def test_self_auth_token_manipulation(self):
        result = self.client.auth.token.create(ttl="1h", renewable=True)
        assert result["auth"]["client_token"]
        self.client.token = result["auth"]["client_token"]

        lookup = self.client.lookup_token(result["auth"]["client_token"])
        assert result["auth"]["client_token"] == lookup["data"]["id"]

        renew = self.client.auth.token.renew_self()
        assert result["auth"]["client_token"] == renew["auth"]["client_token"]

        self.client.revoke_token(lookup["data"]["id"])

        try:
            lookup = self.client.lookup_token(result["auth"]["client_token"])
            assert False
        except exceptions.Forbidden:
            assert True
        except exceptions.InvalidPath:
            assert True
        except exceptions.InvalidRequest:
            assert True

    def test_userpass_auth(self):
        if "userpass/" in self.client.sys.list_auth_methods()["data"]:
            self.client.sys.disable_auth_method("userpass")

        self.client.sys.enable_auth_method("userpass")

        self.client.write(
            "auth/userpass/users/testuser", password="testpass", policies="not_root"
        )

        result = self.client.auth.userpass.login("testuser", "testpass")

        assert self.client.token == result["auth"]["client_token"]
        assert self.client.is_authenticated()

        self.client.token = self.manager.root_token
        self.client.sys.disable_auth_method("userpass")

    def test_create_userpass(self):
        if "userpass/" not in self.client.sys.list_auth_methods()["data"]:
            self.client.sys.enable_auth_method("userpass")

        self.client.auth.userpass.create_or_update_user(
            "testcreateuser", "testcreateuserpass", policies="not_root"
        )

        result = self.client.auth.userpass.login("testcreateuser", "testcreateuserpass")

        assert self.client.token == result["auth"]["client_token"]
        assert self.client.is_authenticated()

        # Test ttl:
        self.client.token = self.manager.root_token
        self.client.auth.userpass.create_or_update_user(
            "testcreateuser", "testcreateuserpass", policies="not_root", ttl="10s"
        )
        self.client.token = result["auth"]["client_token"]

        result = self.client.auth.userpass.login("testcreateuser", "testcreateuserpass")

        assert result["auth"]["lease_duration"] == 10

        self.client.token = self.manager.root_token
        self.client.sys.disable_auth_method("userpass")

    def test_write_data(self):
        self.client.write_data("secret/foo", data={"path": "foo1", "foo": "foo2"})
        result = self.client.read("secret/foo")

        assert result["data"]["path"] == "foo1"
        assert result["data"]["foo"] == "foo2"

        self.client.delete("secret/foo")

    def test_missing_token(self):
        client = utils.create_client()
        assert not client.is_authenticated()

    def test_invalid_token(self):
        client = utils.create_client(token="not-a-real-token")
        assert not client.is_authenticated()

    def test_illegal_token(self):
        client = utils.create_client(token="token-with-new-line\n")
        try:
            client.is_authenticated()
        except ValueError as e:
            assert "Invalid header value" in str(e)

    def test_broken_token(self):
        client = utils.create_client(token="\x1b")
        try:
            client.is_authenticated()
        except exceptions.InvalidRequest as e:
            assert "invalid header value" in str(e)

    def test_client_authenticated(self):
        assert self.client.is_authenticated()

    def test_client_logout(self):
        self.client.logout()
        assert not self.client.is_authenticated()

    def test_client_logout_and_revoke(self):
        # create a new token
        result = self.client.auth.token.create(ttl="1h", renewable=True)
        # set the token
        self.client.token = result["auth"]["client_token"]

        # logout and revoke the token
        self.client.logout(revoke_token=True)
        # set the original token back
        self.client.token = result["auth"]["client_token"]
        # confirm that it no longer is able to authenticate
        assert not self.client.is_authenticated()

    def test_gh51(self):
        key = "secret/http://test.com"

        self.client.write(key, foo="bar")

        result = self.client.read(key)

        assert result["data"]["foo"] == "bar"

    def test_token_accessor(self):
        # Create token, check accessor is provided
        result = self.client.auth.token.create(ttl="1h")
        token_accessor = result["auth"].get("accessor", None)
        assert token_accessor

        # Look up token by accessor, make sure token is excluded from results
        lookup = self.client.lookup_token(token_accessor, accessor=True)
        assert lookup["data"]["accessor"] == token_accessor
        assert not lookup["data"]["id"]

        # Revoke token using the accessor
        self.client.revoke_token(token_accessor, accessor=True)

        # Look up by accessor should fail
        with self.assertRaises(exceptions.InvalidRequest):
            lookup = self.client.lookup_token(token_accessor, accessor=True)

        # As should regular lookup
        with self.assertRaises(exceptions.Forbidden):
            lookup = self.client.lookup_token(result["auth"]["client_token"])

    def test_create_token_explicit_max_ttl(self):
        token = self.client.auth.token.create(ttl="30m", explicit_max_ttl="5m")

        assert token["auth"]["client_token"]

        assert token["auth"]["lease_duration"] == 300

        # Validate token
        lookup = self.client.lookup_token(token["auth"]["client_token"])
        assert token["auth"]["client_token"] == lookup["data"]["id"]

    def test_create_token_max_ttl(self):
        token = self.client.auth.token.create(ttl="5m")

        assert token["auth"]["client_token"]

        assert token["auth"]["lease_duration"] == 300

        # Validate token
        lookup = self.client.lookup_token(token["auth"]["client_token"])
        assert token["auth"]["client_token"] == lookup["data"]["id"]

    def test_create_token_periodic(self):
        token = self.client.auth.token.create(period="30m")

        assert token["auth"]["client_token"]

        assert token["auth"]["lease_duration"] == 1800

        # Validate token
        lookup = self.client.lookup_token(token["auth"]["client_token"])
        assert token["auth"]["client_token"] == lookup["data"]["id"]
        assert lookup["data"]["period"] == 1800

    def test_token_roles(self):
        # Create token role
        assert (
            self.client.auth.token.create_or_update_role("testrole").status_code == 204
        )

        # List token roles
        during = self.client.auth.token.list_roles()["data"]["keys"]
        assert len(during) == 1
        assert during[0] == "testrole"

        # Delete token role
        self.client.auth.token.delete_role("testrole")

    def test_create_token_w_role(self):
        # Create policy
        self.prep_policy("testpolicy")

        # Create token role w/ policy
        assert (
            self.client.auth.token.create_or_update_role(
                "testrole", allowed_policies="testpolicy"
            ).status_code
            == 204
        )

        # Create token against role
        token = self.client.auth.token.create(ttl="1h", role_name="testrole")
        assert token["auth"]["client_token"]
        assert token["auth"]["policies"] == ["default", "testpolicy"]

        # Cleanup
        self.client.auth.token.delete_role("testrole")
        self.client.sys.delete_policy("testpolicy")

    def test_auth_gcp_alternate_mount_point_with_no_client_token_exception(self):
        test_mount_point = "gcp-custom-path"
        # Turn on the gcp backend with a custom mount_point path specified.
        if f"{test_mount_point}/" in self.client.sys.list_auth_methods()["data"]:
            self.client.sys.disable_auth_method(test_mount_point)
        self.client.sys.enable_auth_method("gcp", path=test_mount_point)

        # Drop the client's token to replicate a typical end user's use of any auth method.
        # I.e., its reasonable to expect the method is being called to _retrieve_ a token in the first place.
        self.client.token = None

        # Load a mock JWT stand in for a real document from GCP.
        with open(utils.get_config_file_path("example.jwt")) as fp:
            jwt = fp.read()

        # When attempting to auth (POST) to an auth backend mounted at a different path than the default, we expect a
        # generic 'missing client token' response from Vault.
        expected_exception = (
            exceptions.InvalidRequest
            if utils.vault_version_lt("1.10.0")
            else exceptions.Forbidden
        )
        with self.assertRaises(expected_exception) as assertRaisesContext:
            self.client.auth.gcp.login("example-role", jwt)

        expected_exception_message = (
            "missing client token"
            if utils.vault_version_lt("1.10.0")
            else "permission denied"
        )
        actual_exception_message = str(assertRaisesContext.exception)
        self.assertIn(expected_exception_message, actual_exception_message)

        # Reset test state.
        self.client.token = self.manager.root_token
        self.client.sys.disable_auth_method(path=test_mount_point)

    @skipIf(
        utils.if_vault_version("0.10.0"),
        "KV version 2 secret engine not available before Vault version 0.10.0",
    )
    def test_kv2_secret_backend(self):
        if "test/" in self.client.sys.list_mounted_secrets_engines()["data"]:
            self.client.sys.disable_secrets_engine("test")
        self.client.sys.enable_secrets_engine(
            "kv", path="test", options={"version": "2"}
        )

        secret_backends = self.client.sys.list_mounted_secrets_engines()["data"]

        assert "test/" in secret_backends
        self.assertDictEqual(secret_backends["test/"]["options"], {"version": "2"})

        self.client.sys.disable_secrets_engine("test")

    def test_seal_status(self):
        seal_status_property = self.client.seal_status
        logging.debug("seal_status_property: %s" % seal_status_property)
        self.assertIn(
            member="sealed",
            container=seal_status_property,
        )



from unittest import TestCase

from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestAuth(HvacIntegrationTestCase, TestCase):
    TEST_AUTH_METHOD_TYPE = "github"
    TEST_AUTH_METHOD_PATH = "test-github"

    def tearDown(self):
        self.client.sys.disable_auth_method(path=self.TEST_AUTH_METHOD_PATH)
        super().tearDown()

    def test_auth_backend_manipulation(self):
        self.assertNotIn(
            member="%s/" % self.TEST_AUTH_METHOD_PATH,
            container=self.client.sys.list_auth_methods()["data"],
        )

        self.client.sys.enable_auth_method(
            method_type=self.TEST_AUTH_METHOD_TYPE,
            path=self.TEST_AUTH_METHOD_PATH,
        )
        self.assertIn(
            member="%s/" % self.TEST_AUTH_METHOD_PATH,
            container=self.client.sys.list_auth_methods()["data"],
        )

        self.client.sys.disable_auth_method(
            path=self.TEST_AUTH_METHOD_PATH,
        )
        self.assertNotIn(
            member="%s/" % self.TEST_AUTH_METHOD_PATH,
            container=self.client.sys.list_auth_methods()["data"],
        )

    def test_tune_auth_backend(self):
        test_description = "this is a test auth backend"
        test_max_lease_ttl = 12345678
        self.client.sys.enable_auth_method(
            method_type="approle", path=self.TEST_AUTH_METHOD_PATH
        )

        expected_status_code = 204
        response = self.client.sys.tune_auth_method(
            path=self.TEST_AUTH_METHOD_PATH,
            description=test_description,
            max_lease_ttl=test_max_lease_ttl,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

        response = self.client.sys.read_auth_method_tuning(
            path=self.TEST_AUTH_METHOD_PATH
        )

        self.assertEqual(
            first=test_max_lease_ttl, second=response["data"]["max_lease_ttl"]
        )



from unittest import TestCase

import requests_mock
from parameterized import parameterized

from hvac import Client


class TestApproleRoutes(TestCase):
    """Unit tests providing coverage for approle auth backend-related methods/routes."""

    @parameterized.expand(
        [
            ("default mount point", None, "application1"),
            ("custom mount point", "my-approle-path", "application2"),
        ]
    )
    @requests_mock.Mocker()
    def test_create_role(self, test_label, mount_point, role_name, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.create_or_update_approle(
                role_name=role_name,
            )
        else:
            actual_response = client.auth.approle.create_or_update_approle(
                role_name=role_name,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "application1"),
            ("custom mount point", "my-approle-path", "application2"),
        ]
    )
    @requests_mock.Mocker()
    def test_list_roles(self, test_label, mount_point, role_name, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {
                "keys": [
                    role_name,
                ]
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "e4c219fb-0a78-2be2-8d3c-b3715dccb920",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/role".format(
            "approle" if mount_point is None else mount_point,
        )
        requests_mocker.register_uri(
            method="LIST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.list_roles()
        else:
            actual_response = client.auth.approle.list_roles(
                mount_point=mount_point,
            )

        # ensure we received our mock response data back successfully
        self.assertEqual(mock_response, actual_response)

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "application1",
                "40b3c82d-12a6-838c-9e74-1f1133867e06",
            ),
            (
                "custom mount point",
                "my-approle-path",
                "application2",
                "5fs3c82d-12a6-838c-9e74-1f1133867esf",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_get_role_id(
        self, test_label, mount_point, role_name, role_id, requests_mocker
    ):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {"role_id": role_id},
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "85590a1a-6dd7-de79-01b0-1c285d505bf2",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}/role-id".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.read_role_id(role_name=role_name)
        else:
            actual_response = client.auth.approle.read_role_id(
                role_name=role_name, mount_point=mount_point
            )

        # ensure we received our mock response data back successfully
        self.assertEqual(first=role_id, second=actual_response["data"]["role_id"])

    @parameterized.expand(
        [
            ("default mount point", None, "application1", "custom-role-id-1"),
            (
                "custom mount point",
                "my-approle-path",
                "application2",
                "custom-role-id-2",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_set_role_id(
        self, test_label, mount_point, role_name, role_id, requests_mocker
    ):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}/role-id".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.update_role_id(
                role_name=role_name, role_id=role_id
            )
        else:
            actual_response = client.auth.approle.update_role_id(
                role_name=role_name,
                role_id=role_id,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "application1"),
            ("custom mount point", "my-approle-path", "application2"),
        ]
    )
    @requests_mock.Mocker()
    def test_get_role(self, test_label, mount_point, role_name, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {
                "bind_secret_id": True,
                "bound_cidr_list": "",
                "period": 0,
                "policies": ["default"],
                "secret_id_num_uses": 0,
                "secret_id_ttl": 0,
                "token_max_ttl": 900,
                "token_num_uses": 0,
                "token_ttl": 600,
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "0aab655f-ecd2-b3d4-3817-35b5bdfd3f28",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.read_role(
                role_name=role_name,
            )
        else:
            actual_response = client.auth.approle.read_role(
                role_name=role_name,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=mock_response,
            second=actual_response,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "application1"),
            ("custom mount point", "my-approle-path", "application2"),
        ]
    )
    @requests_mock.Mocker()
    def test_create_role_secret_id(
        self, test_label, mount_point, role_name, requests_mocker
    ):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {
                "secret_id": "be78e3ca-f644-b099-3291-e8a6f5985cfe",
                "secret_id_accessor": "b58fd0ee-130c-33bb-5f69-6d4fd1731e5f",
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "2310dc21-0fea-a2de-2d94-bb4edd59f1e9",
            "warnings": None,
            "wrap_info": None,
        }

        mock_url = "http://localhost:8200/v1/auth/{}/role/{}/secret-id".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.generate_secret_id(
                role_name=role_name,
            )
        else:
            actual_response = client.auth.approle.generate_secret_id(
                role_name=role_name,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=mock_response,
            second=actual_response,
        )

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "application1",
                "be78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
            (
                "custom mount point",
                "my-approle-path",
                "application2",
                "ce78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_get_role_secret_id(
        self, test_label, mount_point, role_name, secret_id, requests_mocker
    ):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {
                "SecretIDNumUses": 0,
                "cidr_list": [],
                "creation_time": "2018-06-11T07:33:57.771908-05:00",
                "expiration_time": "0001-01-01T00:00:00Z",
                "last_updated_time": "2018-06-11T07:33:57.771908-05:00",
                "metadata": {},
                "secret_id_accessor": "b58fd0ee-130c-33bb-5f69-6d4fd1731e5f",
                "secret_id_num_uses": 0,
                "secret_id_ttl": 0,
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "718a00fa-e76f-f1fc-9b9e-f9c4baa766b3",
            "wrap_info": None,
        }

        mock_url = "http://localhost:8200/v1/auth/{}/role/{}/secret-id/lookup".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.read_secret_id(
                role_name=role_name,
                secret_id=secret_id,
            )
        else:
            actual_response = client.auth.approle.read_secret_id(
                role_name=role_name,
                secret_id=secret_id,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=mock_response,
            second=actual_response,
        )

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "application1",
                "be78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
            (
                "custom mount point",
                "my-approle-path",
                "application2",
                "ce78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_list_role_secrets(
        self, test_label, mount_point, role_name, secret_id, requests_mocker
    ):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {"keys": [secret_id]},
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "eb805845-f6ce-a514-9238-6914664dd601",
            "warnings": None,
            "wrap_info": None,
        }

        mock_url = "http://localhost:8200/v1/auth/{}/role/{}/secret-id".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="LIST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.list_secret_id_accessors(
                role_name=role_name,
            )
        else:
            actual_response = client.auth.approle.list_secret_id_accessors(
                role_name=role_name,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=mock_response,
            second=actual_response,
        )

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "application1",
                "be78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
            (
                "custom mount point",
                "my-approle-path",
                "application2",
                "ce78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_get_role_secret_id_accessor(
        self, test_label, mount_point, role_name, secret_id_accessor, requests_mocker
    ):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {
                "SecretIDNumUses": 0,
                "cidr_list": [],
                "creation_time": "2018-06-11T07:33:57.771908-05:00",
                "expiration_time": "0001-01-01T00:00:00Z",
                "last_updated_time": "2018-06-11T07:33:57.771908-05:00",
                "metadata": {},
                "secret_id_accessor": secret_id_accessor,
                "secret_id_num_uses": 0,
                "secret_id_ttl": 0,
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "2c9fcba6-425d-e4c0-45fa-ee90450a3c00",
            "wrap_info": None,
        }

        mock_url = (
            "http://localhost:8200/v1/auth/{}/role/{}/secret-id-accessor/lookup".format(
                "approle" if mount_point is None else mount_point,
                role_name,
            )
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.read_secret_id_accessor(
                role_name=role_name,
                secret_id_accessor=secret_id_accessor,
            )
        else:
            actual_response = client.auth.approle.read_secret_id_accessor(
                role_name=role_name,
                secret_id_accessor=secret_id_accessor,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=mock_response,
            second=actual_response,
        )

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "application1",
                "be78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
            (
                "custom mount point",
                "my-approle-path",
                "application2",
                "ce78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_delete_role_secret_id(
        self, test_label, mount_point, role_name, secret_id, requests_mocker
    ):
        expected_status_code = 204

        mock_url = "http://localhost:8200/v1/auth/{}/role/{}/secret-id/destroy".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.destroy_secret_id(
                role_name=role_name,
                secret_id=secret_id,
            )
        else:
            actual_response = client.auth.approle.destroy_secret_id(
                role_name=role_name,
                secret_id=secret_id,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "application1",
                "be78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
            (
                "custom mount point",
                "my-approle-path",
                "application2",
                "ce78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_delete_role_secret_id_accessor(
        self, test_label, mount_point, role_name, secret_id_accessor, requests_mocker
    ):
        expected_status_code = 204

        mock_url = "http://localhost:8200/v1/auth/{}/role/{}/secret-id-accessor/destroy".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.destroy_secret_id_accessor(
                role_name=role_name,
                secret_id_accessor=secret_id_accessor,
            )
        else:
            actual_response = client.auth.approle.destroy_secret_id_accessor(
                role_name=role_name,
                secret_id_accessor=secret_id_accessor,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "application1",
                "be78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
            (
                "custom mount point",
                "my-approle-path",
                "application2",
                "ce78e3ca-f644-b099-3291-e8a6f5985cfe",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_create_role_custom_secret_id(
        self, test_label, mount_point, role_name, secret_id, requests_mocker
    ):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {
                "secret_id": secret_id,
                "secret_id_accessor": "f5cb4b7d-9111-320e-6f24-73bf45d3845d",
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "e7c8b2e1-95e8-cb17-e98a-6c428201f1d5",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}/custom-secret-id".format(
            "approle" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.create_custom_secret_id(
                role_name=role_name,
                secret_id=secret_id,
            )
        else:
            actual_response = client.auth.approle.create_custom_secret_id(
                role_name=role_name,
                secret_id=secret_id,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=mock_response,
            second=actual_response,
        )

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "c7f93182-c6b1-4b6a-9dfb-03bdb6df0026",
                "26089502-b7d3-412a-b3e6-3d44300f9bd1",
            ),
            (
                "custom mount point",
                "my-approle-path",
                "cf6b7c2e-3866-48f8-a764-3bcb5782a85a",
                "7156c666-0491-4c49-af40-7a97300fbaff",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_auth_approle(
        self, test_label, mount_point, role_id, secret_id, requests_mocker
    ):
        expected_status_code = 200
        mock_response = {
            "auth": {
                "accessor": "f8b576f9-9146-4173-e174-40257d58015a",
                "client_token": "3db3d089-7d3c-f531-cd3e-bfe44696a92c",
                "lease_duration": 600,
                "metadata": {"role_name": "application1"},
                "policies": ["default"],
                "renewable": True,
            },
            "data": None,
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "2eb635ad-a763-926a-9815-4cb4d14a40f9",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/login".format(
            "approle" if mount_point is None else mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        client = Client()
        if mount_point is None:
            actual_response = client.auth.approle.login(
                role_id=role_id,
                secret_id=secret_id,
            )
        else:
            actual_response = client.auth.approle.login(
                role_id=role_id,
                secret_id=secret_id,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=mock_response,
            second=actual_response,
        )



import logging
from unittest import TestCase
from unittest import skipIf

import requests_mock
from parameterized import parameterized

from hvac.adapters import JSONAdapter
from hvac.api.secrets_engines.azure import Azure, DEFAULT_MOUNT_POINT
from tests import utils


@skipIf(
    utils.vault_version_lt("0.11.0"),
    "Azure secret engine not available before Vault version 0.11.0",
)
class TestAzure(TestCase):
    @parameterized.expand(
        [
            ("create role", None),
        ]
    )
    @requests_mock.Mocker()
    def test_create_or_update_role(self, test_label, azure_roles, requests_mocker):
        expected_status_code = 204
        role_name = "hvac"
        if azure_roles is None:
            azure_roles = [
                {
                    "role_name": "Contributor",
                    "scope": "/subscriptions/95e675fa-307a-455e-8cdf-0a66aeaa35ae",
                },
            ]

        mock_url = "http://localhost:8200/v1/{mount_point}/roles/{name}".format(
            mount_point=DEFAULT_MOUNT_POINT,
            name=role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            # json=mock_response,
        )
        azure = Azure(adapter=JSONAdapter())
        create_or_update_role_response = azure.create_or_update_role(
            name=role_name, azure_roles=azure_roles, mount_point=DEFAULT_MOUNT_POINT
        )
        logging.debug(
            "create_or_update_role_response: %s" % create_or_update_role_response
        )

        self.assertEqual(
            first=expected_status_code,
            second=create_or_update_role_response.status_code,
        )

    @parameterized.expand(
        [
            ("some_test",),
        ]
    )
    @requests_mock.Mocker()
    def test_list_roles(self, test_label, requests_mocker):
        expected_status_code = 200
        role_names = ["hvac"]
        mock_response = {
            "data": {
                "roles": role_names,
            },
        }

        mock_url = "http://localhost:8200/v1/{mount_point}/roles".format(
            mount_point=DEFAULT_MOUNT_POINT,
        )
        requests_mocker.register_uri(
            method="LIST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        azure = Azure(adapter=JSONAdapter())
        list_roles_response = azure.list_roles(mount_point=DEFAULT_MOUNT_POINT)
        logging.debug("list_roles_response: %s" % list_roles_response)

        self.assertEqual(
            first=mock_response["data"],
            second=list_roles_response,
        )

    @parameterized.expand(
        [
            ("some_test",),
        ]
    )
    @requests_mock.Mocker()
    def test_generate_credentials(self, test_label, requests_mocker):
        expected_status_code = 200
        role_name = "hvac"
        mock_response = {
            "data": {
                "client_id": "some_client_id",
                "client_secret": "some_client_secret",
            },
        }

        mock_url = "http://localhost:8200/v1/{mount_point}/creds/{name}".format(
            mount_point=DEFAULT_MOUNT_POINT,
            name=role_name,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        azure = Azure(adapter=JSONAdapter())
        generate_credentials_response = azure.generate_credentials(
            name=role_name, mount_point=DEFAULT_MOUNT_POINT
        )
        logging.debug(
            "generate_credentials_response: %s" % generate_credentials_response
        )

        self.assertEqual(
            first=mock_response["data"],
            second=generate_credentials_response,
        )



import json
import logging
from unittest import TestCase

from parameterized import parameterized, param

from hvac import exceptions
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestGcp(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "gcp-test"

    def setUp(self):
        super().setUp()
        if "%s/" % self.TEST_MOUNT_POINT not in self.client.sys.list_auth_methods():
            self.client.sys.enable_auth_method(
                method_type="gcp",
                path=self.TEST_MOUNT_POINT,
            )

    def tearDown(self):
        super().tearDown()
        self.client.sys.disable_auth_method(
            path=self.TEST_MOUNT_POINT,
        )

    @parameterized.expand(
        [
            param(
                "success",
                credentials=utils.load_config_file("example.jwt.json"),
            ),
            param(
                "set invalid credentials",
                credentials="some invalid JWT",
                raises=exceptions.InvalidRequest,
            ),
        ]
    )
    def test_configure(self, label, credentials="", raises=None):
        if raises:
            with self.assertRaises(raises):
                self.client.auth.gcp.configure(
                    credentials=credentials,
                    mount_point=self.TEST_MOUNT_POINT,
                )
        else:
            configure_response = self.client.auth.gcp.configure(
                credentials=credentials,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            self.assertEqual(
                first=bool(configure_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_config(self, label):

        credentials = utils.load_config_file("example.jwt.json")
        self.client.auth.gcp.configure(
            credentials=credentials,
            mount_point=self.TEST_MOUNT_POINT,
        )
        read_config_response = self.client.auth.gcp.read_config(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("read_config_response: %s" % read_config_response)

        creds_dict = json.loads(credentials)
        expected_config = {
            "project_id": creds_dict["project_id"],
            "client_email": creds_dict["client_email"],
            "private_key_id": creds_dict["private_key_id"],
        }
        for k, v in expected_config.items():
            self.assertEqual(
                first=v,
                second=read_config_response[k],
            )

    @parameterized.expand(
        [
            # param(
            #     'success',  # TODO: figure out why this is returning a 405
            # ),
            param(
                "no existing config",
                write_config_first=False,
                raises=exceptions.UnexpectedError,
            ),
        ]
    )
    def test_delete_config(self, label, write_config_first=True, raises=None):

        if write_config_first:
            self.client.auth.gcp.configure(
                mount_point=self.TEST_MOUNT_POINT,
            )
        if raises is not None:
            with self.assertRaises(raises):
                self.client.auth.gcp.delete_config(
                    mount_point=self.TEST_MOUNT_POINT,
                )
        else:
            delete_config_response = self.client.auth.gcp.delete_config(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("delete_config_response: %s" % delete_config_response)
            self.assertEqual(
                first=bool(delete_config_response),
                second=True,
            )

    @parameterized.expand(
        [
            param("success iam", role_type="iam", bound_service_accounts=["*"]),
            param(
                "iam no bound service account",
                role_type="iam",
                raises=exceptions.InvalidRequest,
                exception_message="IAM role type must have at least one service account",
            ),
            param(
                "success gce",
                role_type="gce",
            ),
            param(
                "invalid role type",
                role_type="hvac",
                raises=exceptions.ParamValidationError,
                exception_message="unsupported role_type argument provided",
            ),
        ]
    )
    def test_create_role(
        self,
        label,
        role_type,
        policies=None,
        bound_service_accounts=None,
        raises=None,
        exception_message="",
    ):
        role_name = "hvac"
        project_id = "test-hvac-project-not-a-real-project"
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.gcp.create_role(
                    name=role_name,
                    role_type=role_type,
                    project_id=project_id,
                    policies=policies,
                    bound_service_accounts=bound_service_accounts,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            create_role_response = self.client.auth.gcp.create_role(
                name=role_name,
                role_type=role_type,
                project_id=project_id,
                policies=policies,
                bound_service_accounts=bound_service_accounts,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("create_role_response: %s" % create_role_response)
            self.assertEqual(
                first=bool(create_role_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success add",
                add=["test"],
            ),
            param(
                "success remove",
                remove=["test"],
            ),
            param(
                "fail upon no changes",
                raises=exceptions.InvalidRequest,
                exception_message="must provide at least one value to add or remove",
            ),
            # TODO: wrong role type (gce)
        ]
    )
    def test_edit_service_accounts_on_iam_role(
        self,
        label,
        add=None,
        remove=None,
        create_role_first=True,
        raises=None,
        exception_message="",
    ):
        role_name = "hvac"
        project_id = "test-hvac-project-not-a-real-project"
        if create_role_first:
            self.client.auth.gcp.create_role(
                name=role_name,
                role_type="iam",
                project_id=project_id,
                bound_service_accounts=[
                    "hvac-integration-test@appspot.gserviceaccount.com"
                ],
                mount_point=self.TEST_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.gcp.edit_service_accounts_on_iam_role(
                    name=role_name,
                    add=add,
                    remove=remove,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            edit_sa_on_iam_response = (
                self.client.auth.gcp.edit_service_accounts_on_iam_role(
                    name=role_name,
                    add=add,
                    remove=remove,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            )
            logging.debug("create_role_response: %s" % edit_sa_on_iam_response)
            self.assertEqual(
                first=bool(edit_sa_on_iam_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success add",
                add=["test-key:test-value"],
            ),
            param(
                "success remove",
                remove=["test-key:test-value"],
            ),
            param(
                "fail upon no changes",
                raises=exceptions.InvalidRequest,
                exception_message="must provide at least one value to add or remove",
            ),
            # TODO: wrong role type (iam)
        ]
    )
    def test_edit_labels_on_gce_role(
        self,
        label,
        add=None,
        remove=None,
        create_role_first=True,
        raises=None,
        exception_message="",
    ):
        role_name = "hvac"
        project_id = "test-hvac-project-not-a-real-project"
        if create_role_first:
            self.client.auth.gcp.create_role(
                name=role_name,
                role_type="gce",
                project_id=project_id,
                bound_service_accounts=[
                    "hvac-integration-test@appspot.gserviceaccount.com"
                ],
                mount_point=self.TEST_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.gcp.edit_labels_on_gce_role(
                    name=role_name,
                    add=add,
                    remove=remove,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            edit_labled_response = self.client.auth.gcp.edit_labels_on_gce_role(
                name=role_name,
                add=add,
                remove=remove,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("create_role_response: %s" % edit_labled_response)
            self.assertEqual(
                first=bool(edit_labled_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "nonexistent role",
                create_role_first=False,
                raises=exceptions.InvalidPath,
            ),
        ]
    )
    def test_read_role(
        self, label, create_role_first=True, raises=None, exception_message=""
    ):
        role_name = "hvac"
        project_id = "test-hvac-project-not-a-real-project"
        if create_role_first:
            self.client.auth.gcp.create_role(
                name=role_name,
                role_type="gce",
                project_id=project_id,
                bound_service_accounts=[
                    "hvac-integration-test@appspot.gserviceaccount.com"
                ],
                mount_point=self.TEST_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.gcp.read_role(
                    name=role_name,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            read_role_response = self.client.auth.gcp.read_role(
                name=role_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("create_role_response: %s" % read_role_response)
            if utils.vault_version_ge("1.0.0"):
                returned_project_id = read_role_response["bound_projects"]
                expected_project_id = [project_id]
            else:
                returned_project_id = read_role_response["project_id"]
                expected_project_id = project_id
            self.assertEqual(first=returned_project_id, second=expected_project_id)

    @parameterized.expand(
        [
            param(
                "success one role",
            ),
            param(
                "success multiple roles",
                num_roles_to_create=7,
            ),
            param(
                "no roles",
                num_roles_to_create=0,
                raises=exceptions.InvalidPath,
            ),
        ]
    )
    def test_list_roles(self, label, num_roles_to_create=1, raises=None):
        project_id = "test-hvac-project-not-a-real-project"
        roles_to_create = ["hvac%s" % n for n in range(0, num_roles_to_create)]
        logging.debug("roles_to_create: %s" % roles_to_create)
        for role_to_create in roles_to_create:
            create_role_response = self.client.auth.gcp.create_role(
                name=role_to_create,
                role_type="gce",
                project_id=project_id,
                bound_service_accounts=[
                    "hvac-integration-test@appspot.gserviceaccount.com"
                ],
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("create_role_response: %s" % create_role_response)

        if raises is not None:
            with self.assertRaises(raises):
                self.client.auth.gcp.list_roles(
                    mount_point=self.TEST_MOUNT_POINT,
                )
        else:
            list_roles_response = self.client.auth.gcp.list_roles(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("list_roles_response: %s" % list_roles_response)
            self.assertEqual(
                first=list_roles_response["keys"],
                second=roles_to_create,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
            param(
                "nonexistent role name",
                configure_role_first=False,
            ),
        ]
    )
    def test_delete_role(self, label, configure_role_first=True, raises=None):
        role_name = "hvac"
        project_id = "test-hvac-project-not-a-real-project"
        if configure_role_first:
            create_role_response = self.client.auth.gcp.create_role(
                name=role_name,
                role_type="gce",
                project_id=project_id,
                bound_service_accounts=[
                    "hvac-integration-test@appspot.gserviceaccount.com"
                ],
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("create_role_response: %s" % create_role_response)

        if raises is not None:
            with self.assertRaises(raises):
                self.client.auth.gcp.delete_role(
                    role=role_name,
                    mount_point=self.TEST_MOUNT_POINT,
                )
        else:
            delete_role_response = self.client.auth.gcp.delete_role(
                role=role_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("delete_role_response: %s" % delete_role_response)
            self.assertEqual(
                first=bool(delete_role_response),
                second=True,
            )



import logging
from unittest import TestCase
from unittest import skipIf

import requests_mock
from parameterized import parameterized

from hvac.adapters import JSONAdapter
from hvac.api.auth_methods import Kubernetes
from tests import utils


@skipIf(
    utils.vault_version_lt("0.8.3"),
    "Kubernetes auth method not available before Vault version 0.8.3",
)
class TestKubernetes(TestCase):
    TEST_MOUNT_POINT = "kubernetes-test"

    @parameterized.expand(
        [
            (
                "success",
                dict(),
                None,
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_login(self, label, test_params, raises, requests_mocker):
        role_name = "hvac"
        test_policies = [
            "default",
            "dev",
            "prod",
        ]
        expected_status_code = 200
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/login".format(
            mount_point=self.TEST_MOUNT_POINT,
        )
        mock_response = {
            "auth": {
                "client_token": "38fe9691-e623-7238-f618-c94d4e7bc674",
                "accessor": "78e87a38-84ed-2692-538f-ca8b9f400ab3",
                "policies": test_policies,
                "metadata": {
                    "role": role_name,
                    "service_account_name": "vault-auth",
                    "service_account_namespace": "default",
                    "service_account_secret_name": "vault-auth-token-pd21c",
                    "service_account_uid": "aa9aa8ff-98d0-11e7-9bb7-0800276d99bf",
                },
                "lease_duration": 2764800,
                "renewable": True,
            },
        }
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        kubernetes = Kubernetes(adapter=JSONAdapter())
        if raises is not None:
            with self.assertRaises(raises):
                kubernetes.login(
                    role=role_name,
                    jwt="my-jwt",
                    mount_point=self.TEST_MOUNT_POINT,
                    **test_params
                )
        else:
            login_response = kubernetes.login(
                role=role_name,
                jwt="my-jwt",
                mount_point=self.TEST_MOUNT_POINT,
                **test_params
            )
            logging.debug("login_response: %s" % login_response)
            self.assertEqual(
                first=login_response["auth"]["policies"],
                second=test_policies,
            )

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "127.0.0.1:80",
                ["-----BEGIN CERTIFICATE-----test_key-----END CERTIFICATE-----"],
            ),
            (
                "custom mount point",
                "k8s",
                "some_k8s_host.com",
                ["-----BEGIN CERTIFICATE-----test_key-----END CERTIFICATE-----"],
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_configure(
        self, test_label, mount_point, kubernetes_host, pem_keys, requests_mocker
    ):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/config".format(
            "kubernetes" if mount_point is None else mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        kubernetes = Kubernetes(adapter=JSONAdapter())

        test_arguments = dict(
            kubernetes_host=kubernetes_host,
            pem_keys=pem_keys,
        )
        if mount_point:
            test_arguments["mount_point"] = mount_point

        actual_response = kubernetes.configure(**test_arguments)

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", None),
            ("custom mount point", "k8s"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_configuration(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {
                "kubernetes_ca_cert": "",
                "kubernetes_host": "127.0.0.1:80",
                "pem_keys": ["some key"],
                "token_reviewer_jwt": "",
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "12687b5f-b4f5-2ba4-aae2-2a8d7e53ca55",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/config".format(
            "kubernetes" if mount_point is None else mount_point,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        kubernetes = Kubernetes(adapter=JSONAdapter())

        test_arguments = dict()
        if mount_point:
            test_arguments["mount_point"] = mount_point

        actual_response = kubernetes.read_config(**test_arguments)

        self.assertEqual(
            first=mock_response["data"],
            second=actual_response,
        )

    @parameterized.expand(
        [
            (
                "default mount point",
                None,
                "application1",
                "*",
                "some-namespace",
                "serviceaccount_uid",
            ),
            (
                "custom mount point",
                "k8s",
                "application2",
                "some-service-account",
                "*",
                "serviceaccount_name",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_create_role(
        self,
        test_label,
        mount_point,
        role_name,
        bound_service_account_names,
        bound_service_account_namespaces,
        alias_name_source,
        requests_mocker,
    ):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}".format(
            "kubernetes" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        kubernetes = Kubernetes(adapter=JSONAdapter())

        test_arguments = dict(
            name=role_name,
            bound_service_account_names=bound_service_account_names,
            bound_service_account_namespaces=bound_service_account_namespaces,
            alias_name_source=alias_name_source,
        )
        if mount_point:
            test_arguments["mount_point"] = mount_point
        actual_response = kubernetes.create_role(**test_arguments)

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "application1"),
            ("custom mount point", "k8s", "application2"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_role(self, test_label, mount_point, role_name, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {
                "bind_secret_id": True,
                "bound_cidr_list": "",
                "period": 0,
                "policies": ["default"],
                "secret_id_num_uses": 0,
                "secret_id_ttl": 0,
                "token_max_ttl": 900,
                "token_num_uses": 0,
                "token_ttl": 600,
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "0aab655f-ecd2-b3d4-3817-35b5bdfd3f28",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}".format(
            "kubernetes" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        kubernetes = Kubernetes(adapter=JSONAdapter())

        test_arguments = dict(
            name=role_name,
        )
        if mount_point:
            test_arguments["mount_point"] = mount_point

        actual_response = kubernetes.read_role(**test_arguments)

        self.assertEqual(
            first=mock_response["data"],
            second=actual_response,
        )

    @parameterized.expand(
        [
            ("default mount point", None, ["test-role-1", "test-role-2"]),
            ("custom mount point", "k8s", ["test-role"]),
        ]
    )
    @requests_mock.Mocker()
    def test_list_roles(self, test_label, mount_point, role_names, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {
                "keys": role_names,
            },
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "e4c219fb-0a78-2be2-8d3c-b3715dccb920",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/role".format(
            "kubernetes" if mount_point is None else mount_point,
        )
        requests_mocker.register_uri(
            method="LIST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        kubernetes = Kubernetes(adapter=JSONAdapter())

        test_arguments = dict()
        if mount_point:
            test_arguments["mount_point"] = mount_point
        actual_response = kubernetes.list_roles(**test_arguments)

        # ensure we received our mock response data back successfully
        self.assertEqual(mock_response["data"], actual_response)

    @parameterized.expand(
        [
            ("default mount point", None, "application1"),
            ("custom mount point", "k8s", "application2"),
        ]
    )
    @requests_mock.Mocker()
    def test_delete_role(self, test_label, mount_point, role_name, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}".format(
            "kubernetes" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="DELETE",
            url=mock_url,
            status_code=expected_status_code,
        )
        kubernetes = Kubernetes(adapter=JSONAdapter())

        test_arguments = dict(
            name=role_name,
        )
        if mount_point:
            test_arguments["mount_point"] = mount_point

        actual_response = kubernetes.delete_role(**test_arguments)

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )



import logging
from unittest import TestCase, skipIf

from parameterized import parameterized, param

from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase
from tests import utils


class TestPki(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "ssh-integration-test"
    PRIVATE_SSH_KEY = "tests/config_files/ssh-key"
    PUBLIC_SSH_KEY = "tests/config_files/ssh-key.pub"

    def setUp(self):
        super().setUp()
        self.client.sys.enable_secrets_engine(
            backend_type="ssh",
            path=self.TEST_MOUNT_POINT,
        )

    def tearDown(self):
        self.client.sys.disable_secrets_engine(path=self.TEST_MOUNT_POINT)
        super().tearDown()

    # TODO: deprecate all dynamic SSH keys methods from hvac
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    @skipIf(
        utils.vault_version_ge("1.13.0"), reason="Vault 1.13.0 dropped this feature."
    )
    def test_create_key(self, label, raises=False, exception_message=""):
        with open(self.PRIVATE_SSH_KEY) as key_file:
            private_key = key_file.read()

        create_key_response = self.client.secrets.ssh.create_or_update_key(
            name="test-key",
            key=private_key,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_or_update_key_response: %s" % create_key_response)
        self.assertEqual(
            first=create_key_response.status_code,
            second=204,
        )

    # TODO: deprecate all dynamic SSH keys methods from hvac
    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    @skipIf(
        utils.vault_version_ge("1.13.0"), reason="Vault 1.13.0 dropped this feature."
    )
    def test_delete_key(self, label, raises=False, exception_message=""):
        with open(self.PRIVATE_SSH_KEY) as key_file:
            private_key = key_file.read()

        key_name = "test-key"
        self.client.secrets.ssh.create_or_update_key(
            name=key_name,
            key=private_key,
            mount_point=self.TEST_MOUNT_POINT,
        )

        delete_key_response = self.client.secrets.ssh.delete_key(
            name=key_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("delete_key_response: %s" % delete_key_response)
        self.assertEqual(
            first=delete_key_response.status_code,
            second=204,
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_create_role(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"

        create_role_response = self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("create_role_response: %s" % create_role_response)
        self.assertEqual(first=create_role_response.status_code, second=204)

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_role(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )

        read_role_response = self.client.secrets.ssh.read_role(
            name=role_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        print(read_role_response)
        logging.debug("read_role_response: %s" % read_role_response)
        self.assertEqual(
            first=read_role_response["data"],
            second={
                "allowed_users": "",
                "cidr_list": "",
                "default_user": "root",
                "exclude_cidr_list": "",
                "key_type": "otp",
                "port": 22,
            },
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_list_roles(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )

        list_roles_response = self.client.secrets.ssh.list_roles(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("list_roles_response: %s" % list_roles_response)
        self.assertEqual(
            first=list_roles_response["data"],
            second={
                "key_info": {
                    "test_role": {
                        "key_type": "otp",
                    }
                },
                "keys": [
                    "test_role",
                ],
            },
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_delete_role(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )

        delete_role_response = self.client.secrets.ssh.delete_role(
            name=role_name,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("delete_role_response: %s" % delete_role_response)
        self.assertEqual(first=delete_role_response.status_code, second=204)

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_configure_zeroaddress_roles(
        self, label, raises=False, exception_message=""
    ):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )

        configure_zeroaddress_roles_response = (
            self.client.secrets.ssh.configure_zeroaddress_roles(
                roles=role_name,
                mount_point=self.TEST_MOUNT_POINT,
            )
        )

        logging.debug(
            "configure_zeroaddress_roles_response: %s"
            % configure_zeroaddress_roles_response
        )
        self.assertEqual(
            first=configure_zeroaddress_roles_response.status_code, second=204
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_list_zeroaddress_roles(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.client.secrets.ssh.configure_zeroaddress_roles(
            roles=role_name,
            mount_point=self.TEST_MOUNT_POINT,
        )

        list_zeroaddress_roles_response = (
            self.client.secrets.ssh.list_zeroaddress_roles(
                mount_point=self.TEST_MOUNT_POINT,
            )
        )

        logging.debug(
            "list_zeroaddress_roles_response: %s" % list_zeroaddress_roles_response
        )
        self.assertEqual(
            first=list_zeroaddress_roles_response["data"],
            second={"roles": ["test_role"]},
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_delete_zeroaddress_roles(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.client.secrets.ssh.configure_zeroaddress_roles(
            roles=role_name,
            mount_point=self.TEST_MOUNT_POINT,
        )

        delete_zeroaddress_roles_response = (
            self.client.secrets.ssh.delete_zeroaddress_role(
                mount_point=self.TEST_MOUNT_POINT,
            )
        )

        logging.debug(
            "delete_zeroaddress_roles_response: %s" % delete_zeroaddress_roles_response
        )
        self.assertEqual(
            first=delete_zeroaddress_roles_response.status_code, second=204
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_generate_ssh_credentials(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"
        ip = "1.2.3.4"

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.client.secrets.ssh.configure_zeroaddress_roles(
            roles=role_name,
            mount_point=self.TEST_MOUNT_POINT,
        )

        generate_ssh_creds_response = self.client.secrets.ssh.generate_ssh_credentials(
            name=role_name,
            username=default_user,
            ip=ip,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("generate_ssh_creds_response: %s" % generate_ssh_creds_response)
        self.assertEqual(first=generate_ssh_creds_response["data"]["ip"], second=ip)

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_list_roles_by_ip(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"
        ip = "1.2.3.4"

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.client.secrets.ssh.configure_zeroaddress_roles(
            roles=role_name,
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.client.secrets.ssh.generate_ssh_credentials(
            name=role_name,
            username=default_user,
            ip=ip,
            mount_point=self.TEST_MOUNT_POINT,
        )

        list_roles_by_ip_response = self.client.secrets.ssh.list_roles_by_ip(
            ip=ip,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("generate_ssh_creds_response: %s" % list_roles_by_ip_response)
        self.assertEqual(
            first=list_roles_by_ip_response["data"]["roles"], second=[role_name]
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_verify_ssh_otp(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "otp"
        ip = "1.2.3.4"

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.client.secrets.ssh.configure_zeroaddress_roles(
            roles=role_name,
            mount_point=self.TEST_MOUNT_POINT,
        )

        generate_ssh_creds_response = self.client.secrets.ssh.generate_ssh_credentials(
            name=role_name,
            username=default_user,
            ip=ip,
            mount_point=self.TEST_MOUNT_POINT,
        )
        key = generate_ssh_creds_response["data"]["key"]

        verify_ssh_otp_response = self.client.secrets.ssh.verify_ssh_otp(
            otp=key,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("verify_ssh_otp_response: %s" % verify_ssh_otp_response)
        self.assertEqual(
            first=verify_ssh_otp_response["data"]["ip"],
            second=ip,
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_submit_ca_information(self, label, raises=False, exception_message=""):
        submit_ca_information_response = self.client.secrets.ssh.submit_ca_information(
            generate_signing_key=True,
            mount_point=self.TEST_MOUNT_POINT,
        )

        logging.debug(
            "submit_ca_information_response: %s" % submit_ca_information_response
        )
        self.assertIsInstance(
            obj=submit_ca_information_response["data"]["public_key"],
            cls=str,
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_delete_ca_information(self, label, raises=False, exception_message=""):
        self.client.secrets.ssh.submit_ca_information(
            generate_signing_key=True,
            mount_point=self.TEST_MOUNT_POINT,
        )

        delete_ca_information_response = self.client.secrets.ssh.delete_ca_information(
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug(
            "delete_ca_information_response: %s" % delete_ca_information_response
        )
        self.assertEqual(
            first=delete_ca_information_response.status_code,
            second=204,
        )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_sign_ssh_key(self, label, raises=False, exception_message=""):
        role_name = "test_role"
        default_user = "root"
        key_type = "ca"

        self.client.secrets.ssh.submit_ca_information(
            generate_signing_key=True,
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.client.secrets.ssh.create_role(
            name=role_name,
            default_user=default_user,
            key_type=key_type,
            allow_user_certificates=True,
            mount_point=self.TEST_MOUNT_POINT,
        )

        with open(self.PUBLIC_SSH_KEY) as key_file:
            public_key = key_file.read()

        read_public_key_response = self.client.secrets.ssh.sign_ssh_key(
            name=role_name,
            public_key=public_key,
            mount_point=self.TEST_MOUNT_POINT,
        )
        logging.debug("read_public_key_response: %s" % read_public_key_response)
        self.assertIsInstance(
            obj=read_public_key_response["data"]["signed_key"],
            cls=str,
        )
